
# Copyright (c) 2002 Massachusetts Institute of Technology,
# All rights reserved.
# 
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose, without fee, and without
# written agreement is hereby granted, provided that the above
# copyright notice and the following two paragraphs appear in all
# copies of this software.
# 
# IN NO EVENT SHALL THE MASSACHUSETTS INSTITUTE OF TECHNOLOGY BE
# LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
# CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE AND
# ITS DOCUMENTATION, EVEN IF THE MASSACHUSETTS INSTITUTE OF
# TECHNOLOGY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# THE MASSACHUSETTS INSTITUTE OF TECHNOLOGY SPECIFICALLY DISCLAIMS
# ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE
# SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE
# MASSACHUSETTS INSTITUTE OF TECHNOLOGY HAS NO OBLIGATION TO
# PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
# 
# Code contributed by Ronny Krashinsky, ronny@mit.edu, http://www.cag.lcs.mit.edu/~ronny


source pslink.tcl
source http.tcl

set opt(trsplit) "-"
set opt(tr)	""
set opt(namtr)	""
#set opt(stop)	100
set opt(seed)	0

set opt(pages) 100

set opt(ps_tr) 0

set opt(psm) 0
set opt(stayalive)	0
set opt(li_max) 8
set opt(li_func) li_null
set opt(li_percent) 0
set opt(bp) 100

set opt(mb_bw)	  5Mb 
set opt(mb_delay) 0.1ms
set opt(bs_bw)	  10Mb 
set opt(bs_delay) 20ms

#set opt(tcp)	TCP/Reno
#set opt(sink)	TCPSink
set opt(tcp)	TCP/FullTcp
set opt(sink)	TCP/FullTcp

set opt(pktsize) 1460
set opt(maxwin) 20

set opt(webModel) ./cdf_data
set opt(maxConn) 4
set opt(phttp) 0

proc getopt {argc argv} {
    global opt
    for {set i 0} {$i < $argc} {incr i} {
        set tmp [lindex $argv $i]
        if {[string range $tmp 0 0] != "-"} continue
        set name [string range $tmp 1 end]
        if {[info exists opt($name)]} {
            set val [lindex $argv [incr i]]
            global opt($name)
            set opt($name) $val
            puts "-$name set to $opt($name)"
        } else {
            puts "Invalid option: $name"
	    exit
        }       
    }

}


proc finish {} {
	global env nshome pwd
	global ns opt trfd http pslink

	$ns flush-trace
	if {$opt(tr) != ""} {
	    close $trfd
	}

    $http(0) finish
    if {$opt(psm)} {
	$pslink finish
    }

	exit 0
}


proc create-trace {} {
	global ns opt

	if {$opt(tr) != ""} {
	    if [file exists $opt(tr)] {
		exec touch $opt(tr).
		eval exec rm -f $opt(tr) $opt(tr)-bw [glob $opt(tr).*]
	    }
	    set trfd [open $opt(tr) w]
	    $ns trace-all $trfd
	} else {
	    set trfd -1
	}	

	if {$opt(namtr) != ""} {
		$ns namtrace-all [open $opt(namtr) w]
	}
	return $trfd
}


proc create-mobile-topology {} {
    global ns opt mobile server pslink tr_base
    
    set mobile [$ns node]
    set base_station [$ns node]
    set server [$ns node]
    
    $ns duplex-link $mobile $base_station $opt(mb_bw) $opt(mb_delay) DropTail
    $ns duplex-link $base_station $server $opt(bs_bw) $opt(bs_delay) DropTail
    $ns duplex-link-op $mobile $base_station queuePos 0.5

    if {$opt(psm)} {
	set pslinkopt "-tr_base $tr_base -stayalive $opt(stayalive) -listen_interval_max $opt(li_max) \
		-listen_interval_func $opt(li_func) -listen_interval_percent $opt(li_percent) \
		-beacon_period_ms $opt(bp) -trace $opt(ps_tr)"
	set pslink [eval new PSLink $ns $mobile $base_station $pslinkopt]
    }    

    $mobile label MobileClient
    $base_station label BaseStation
    $server label Server
}


proc create-source {} {
	global ns opt mobile server

	if {$opt(webModel) != ""} {
		create-webModel $opt(webModel)
	}

	new_http 0 $server $mobile
}

proc create-webModel path {
#	Http setCDF rvClientTime 0
#	Http setCDF rvClientTime $path/HttpThinkTime.cdf
	Http setCDF rvClientTime $path/HttpThinkTime.1000.cdf
	Http setCDF rvReqLen $path/HttpRequestLength.cdf
	Http setCDF rvNumImg $path/HttpConnections.cdf
	set rv [Http setCDF rvRepLen $path/HttpReplyLength.cdf]
	Http setCDF rvImgLen $rv
    Http setCDF rvServerTime $path/HttpServerTime.cdf
}

proc new_http {i server client} {
	global ns opt http webm tr_base
	set webopt "-srcType $opt(tcp) -snkType $opt(sink) -phttp $opt(phttp) \
		-maxConn $opt(maxConn) -tr_base $tr_base -max_page $opt(pages)"
	set http($i) [eval new Http $ns $client $server $webopt]
	$ns at [expr 0 + $i/1000.0] "$http($i) start"
}


## MAIN ##
getopt $argc $argv
if {$opt(seed) >= 0} { ns-random $opt(seed) }
if [info exists opt(qsize)] { Queue set limit_ $opt(qsize) }

set tr_base data/http-seed$opt(seed)-psm$opt(psm)-bp$opt(bp)-sa$opt(stayalive)-$opt(li_func)-pct$opt(li_percent)-max$opt(li_max)-bs$opt(bs_delay)

set ns [new Simulator]
set trfd [create-trace]

Agent/TCP/FullTcp set segsize_ $opt(pktsize)
Agent/TCP/FullTcp set window_ $opt(maxwin)

#create-topology
create-mobile-topology
#$lan trace $ns $trfd

create-source
#$ns at $opt(stop) "finish"
$ns run
